# -*- coding: utf-8 -*-
"""
    __init__

    Generic tests for nereid.

    :copyright: (c) 2011-2012 by Openlabs Technologies & Consulting (P) Limited
    :license: GPLv3, see LICENSE for more details.
"""
from texttestrunner import suite
