User's Guide
------------

This part of the documentation, which is mostly prose, begins with some
background information about Nereid, then focuses on step-by-step
instructions for web development with Nereid.

Familiarity with Tryton is assumed.

.. toctree::
   :maxdepth: 2

   foreword
   advanced_foreword
   installation
   quickstart
   tutorial


API Reference
-------------

If you are looking for information on a specific function, class or
method, this part of the documentation is for you.

.. toctree::
   :maxdepth: 2

   api
   tryton_module_api


Additional Notes
----------------

Design notes, legal information and changelog are here for the interested.

.. toctree::
   :maxdepth: 2

   versionmanagement
   license
